#include <PaSoRi.h>

//#define DEBUG

#define DEVADDR 1
#define CONFVALUE 1

/* Initialize device */
const byte PASORI_INIT0[] = {0x62,0x01,0x82};
const byte PASORI_INIT1[] = {0x62,0x02,0x80,0x81}; /* INIT3 */
const byte PASORI_INIT2[] = {0x62,0x22,0x80,0xcc,0x81,0x88};
const byte PASORI_INIT3[] = {0x62,0x02,0x80,0x81}; /* INIT1 */
const byte PASORI_INIT4[] = {0x62,0x02,0x82,0x87};
const byte PASORI_INIT5[] = {0x62,0x21,0x25,0x58};
const byte PASORI_READ2[] = {0x5a,0x80};

byte PaSoRi::begin()
{
  Max.powerOn();
  delay(500);
  return init();
}

byte PaSoRi::init()
{
  do {
    task();
  } while (Usb.getUsbTaskState() != USB_STATE_CONFIGURING);
  
  byte rcode;  //return code
  ep_record[0] = *( Usb.getDevTableEntry( 0,0 ));  //copy endpoint 0 parameters
  ep_record[1].epAddr = 0x01;
  ep_record[1].Attr  = 0x03;
  ep_record[1].MaxPktSize = 8;
  ep_record[1].Interval  = 0x01;
  ep_record[1].sndToggle = bmSNDTOG0;
  ep_record[1].rcvToggle = bmRCVTOG0;
  Usb.setDevTableEntry( DEVADDR, ep_record );

  /* read the device descriptor and check VID and PID*/
  byte buf[DEV_DESCR_LEN];
  rcode = Usb.getDevDescr(DEVADDR, ep_record[0].epAddr, DEV_DESCR_LEN, (char*)buf);
  if (rcode)
    return rcode;
#ifdef DEBUG
  Serial.print(buf[8], HEX);
  Serial.println(buf[9], HEX);
  Serial.print(buf[10], HEX);
  Serial.println(buf[11], HEX);
#endif
  if((buf[8]  != PASORI_VID_LO) || (buf[9]  != PASORI_VID_HI) ||
     (buf[10] != PASORI_PID_LO) || (buf[11] != PASORI_PID_HI)) {
       return 0xff;
  }

  /* Configure device */
  rcode = Usb.setConf(DEVADDR, 0, CONFVALUE);
  if (rcode)
    return rcode;
  Usb.setUsbTaskState(USB_STATE_RUNNING);
  
  if (!rcode) rcode = send_packet(sizeof(PASORI_INIT0), PASORI_INIT0);
  if (!rcode) send_packet(sizeof(PASORI_INIT1), PASORI_INIT1);
  if (!rcode) send_packet(sizeof(PASORI_INIT2), PASORI_INIT2);
  if (!rcode) send_packet(sizeof(PASORI_INIT3), PASORI_INIT3);
  if (!rcode) send_packet(sizeof(PASORI_INIT4), PASORI_INIT4);
  if (!rcode) send_packet(sizeof(PASORI_INIT5), PASORI_INIT5);
  if (!rcode) send_packet(sizeof(PASORI_READ2), PASORI_READ2);
  return rcode;
}

void PaSoRi::task()
{
  Max.Task();
  Usb.Task();
}

byte PaSoRi::poll(unsigned short syscode, byte rfu, byte timeslot)
{

  byte cmd[] = {PASORI2_CMD_SEND_PACKET, 0x06, FELICA_CMD_POLLING,
                syscode>>8, syscode&0xff, rfu, timeslot};
  byte rcode = send_packet(sizeof(cmd), cmd);
  if (rcode == 0) {
    byte buf[64];
    rcode = recv(sizeof(buf), buf);
    if (rcode == 0) {
      byte size = buf[3]-1;
      if (size < 16) return 0xff;
      if (buf[7] != FELICA_ANS_POLLING) return 0xfe;
      memcpy(idm, buf+8,  8);
      memcpy(pmm, buf+16, 8);
    }
  }
  return rcode;
}

int PaSoRi::read_without_encryption02(int servicecode, int mode, byte addr, byte b[16])
{
  byte buf[64];
  int i;
  
  buf[0] = PASORI2_CMD_SEND_PACKET;
  buf[1] = 16; // size
  buf[2] = FELICA_CMD_READ_WITHOUT_ENCRYPTION;
  memcpy(&buf[3],idm,8);
  buf[11] = 1;
  buf[12] = servicecode & 0xff;
  buf[13] = servicecode >> 8;
  buf[14] = 1;
  buf[15] = 0x80; // blacklist[0], element size = 2
  buf[16] = addr; // blacklist[1]
  
  byte rcode = send_packet(17, buf);
  if (rcode) return rcode;
  rcode = recv(sizeof(buf), buf);
  if (rcode) return rcode;
  if (buf[5] != PASORI2_ANS_SEND_PACKET ||
      buf[7] != FELICA_ANS_READ_WITHOUT_ENCRYPTION)
    return 0xfe;
  if (buf[16] != 0)
    return buf[16]*256 + buf[17]; // error code
  memcpy(b,&buf[19],16);
  return 0;
}

byte PaSoRi::mobile_felica_push_url(int len, const char *url)
{
  byte buf[64];
  int i;

  buf[0] = PASORI2_CMD_SEND_PACKET;
  buf[1] = len+19; // size
  buf[2] = FELICA_CMD_PUSH;
  memcpy(&buf[3],idm,8);
  buf[11] = len + 8;
  buf[12] = 0x01;    // nr_params
  buf[13] = 0x02;    // launch browser
  buf[14] = len + 2; // param size (LO)
  buf[15] = 0x00;    // param size (HI)
  buf[16] = len;     // URL size (LO)
  buf[17] = 0x00;    // URL size (LO)
  memcpy(&buf[18],url,len);
  unsigned short sum = 0;
  for (int i = 12; i < len+18; i++)
    sum += buf[i];
  sum = 0x10000 - sum;
  buf[len+18] = sum >> 8;
  buf[len+19] = sum & 0xff;

  byte rcode = send_packet(len+20, buf);
  if (rcode) return rcode;
  rcode = recv(sizeof(buf), buf);
  if (rcode) return rcode;
  if (buf[5] != PASORI2_ANS_SEND_PACKET || buf[7] != FELICA_ANS_PUSH)
    return 0xfe;
  return rcode;
}


byte PaSoRi::send_packet(int len, const byte *data)
{
  byte buf[128];
  buf[0] = 0;
  buf[1] = 0;
  buf[2] = 0xff;
  buf[3] = len;
  buf[4] = 0x100 - len;
  memcpy(&buf[5],data,len);
  byte sum = 0;
  for (int i = 0; i < len; i++)
    sum += data[i];
  buf[len+5] = 0x100 - sum;
  byte rcode = send(len+7, buf);
  if (rcode)
    return rcode;

  rcode = recv(sizeof(buf), buf); // recv ack
  return rcode;
}


// low level transmission

byte PaSoRi::send(int len, byte *buf)
{
  byte rcode = Usb.ctrlReq(DEVADDR, ep_record[0].epAddr,
                     USB_SETUP_HOST_TO_DEVICE|USB_SETUP_TYPE_VENDOR,
                     0, 0, 0, 0, len, (char*)buf);
#ifdef DEBUG
  if (rcode) {
    Serial.print("send failed: ret = ");
    Serial.println(rcode, HEX);
  }
  Serial.print(" << ");
  for (int i = 0; i < len; i++) {
    Serial.print((byte)buf[i], HEX);
    Serial.print(" ");
  }
  Serial.print("\r\n");
#endif
  return rcode;
}

byte PaSoRi::recv(int len, byte *buf)
{
  byte rcode = Usb.inTransfer(DEVADDR, ep_record[1].epAddr, len, (char*)buf, 2000);
#ifdef DEBUG
  if (rcode) {
    Serial.print("recv failed: ret = ");
    Serial.println(rcode, HEX);
  }
  Serial.print(" >> ");
  for (int i = 0; i < 32; i++) {
    Serial.print((byte)buf[i], HEX);
    Serial.print(" ");
  }
  Serial.print("\r\n");
#endif
  return rcode;
}
